﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fitter;

namespace DMFitterTestCS
{
    public partial class MainForm : Form
    {
        public DMFitter MyFitter;
        public MainForm()
        {
            InitializeComponent();
            MyFitter=new DMFitter();
            MyFitter.OnProgress += new IDMFitterEvents_OnProgressEventHandler(MyFitter_OnProgress);
            MyFitter.OnGetLMFunction += new IDMFitterEvents_OnGetLMFunctionEventHandler(MyFitter_OnGetLMFunction);
            MyFitter.OnGetLMDerivative+=new IDMFitterEvents_OnGetLMDerivativeEventHandler(MyFitter_OnGetLMDerivative);
        }

        private void CloseButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FitButton_Click(object sender, EventArgs e)
        {
            // create fitting data
            double[] X = new double[100]; // 0..99, 100 elements!
            double[] Y = new double[100];
            for (int i = 0; i < X.Length; i++) 
            {
                X[i] = i;
                Y[i] = 0.123 + 3.456 * i - 0.345 * i*i + 0.00567 * i*i*i;
            };
            // initialize parameter arrays
            double[] Par = new double[4];
            double[] Sig = new double[4];
            double[] Opt = new double[3];
            for (int i = 0; i < Par.Length; i++)
            {
                Par[i] = 1.0;
                Sig[i] = 0.0;
            };
            //MessageBox.Show(Par.Length.ToString());
            //return;
            Opt[0] = 0.0000001;
            Opt[1] = 0.0000001;
            Opt[2] = 0.0;
            // initialize fitter
            if (ParserRadioButton.Checked) {MyFitter.Expression = "p1+p2*cx+p3*cx^2+p4*cx^3";} else 
            {
                MyFitter.Expression = -1; // <- means using both events
                if (FuncRadioButton.Checked)
                { MyFitter.Expression = -2; }; // <- means using ONLY OnGetLMFunction
            };
            //MessageBox.Show(MyFitter.Expression); wrong!
            //MessageBox.Show(MyFitter.Expression.ToString()); ok!
            MyFitter.X = X;
            MyFitter.Y = Y;
            MyFitter.ParamCount = 4;
            MyFitter.Parameters = Par;
            MyFitter.Sigmas = Sig;
            MyFitter.WeightType = 0;
            MyFitter.Options = Opt;
            MyFitter.Iterations = 50;
            // fitting
            if (MyFitter.LMFit()) 
            {
                MessageBox.Show("Deviation: " + MyFitter.Deviation + 
                    "\r\nResult code: " + MyFitter.ResultCode);
            } else
            {
                MessageBox.Show("Error code: " + MyFitter.ResultCode);
            };
        }

        // manually created event handlers

        private void MyFitter_OnProgress()
        {
            double[] Par = new double[4];
            Par = (double[])MyFitter.Parameters;
            ParametersListBox.Items.Clear();
            for (int i = 0; i < Par.Length; i++)
            {
                ParametersListBox.Items.Add("P" + (i + 1) + "=" + Par[i]);
            };
            ParametersListBox.Refresh();
        }

        private double MyFitter_OnGetLMFunction(object X, object Parameters)
        {
            //double[] Par = new double[4]; wrong!
            object[] Par = new object[4];
            double x;
            x = (double)X;
            // Par = (double[])Parameters; wrong!
            Par = (object[])Parameters;
            //return Par[0] + Par[1] * x + Par[2] * x * x + Par[3] * x * x * x; wrong!
            return (double)Par[0] + (double)Par[1] * x +
                (double)Par[2] * x * x + (double)Par[3] * x * x * x;
        }

        private object MyFitter_OnGetLMDerivative(object X, object Parameters, object Sigmas)
        {
            object[] Result = new object[4];
            double x;
            x = (double)X;
            Result[0] = 1.0;
            Result[1] = x;
            Result[2] = x*x;
            Result[3] = x*x*x;
            return Result;
        }
    }
}