program TestLMFitEvents;
(*
This Delphi application is a very close translation
of VBScript code from TestLMFitEvents.htm sample.

Notice that I have to implement special helper class 
in order to connect to COM events. Standard TLB import 
and TOLEControl wrapper cannot be used because DMFitter 
implements very limited subset of OLE interfaces.
*)

{$APPTYPE CONSOLE}

uses
  SysUtils, COMObj, Variants, ActiveX, Windows;

{ Event sink helper coclass }

type
  TTestFitterEventSink = class(TInterfacedObject, IInterface, IDispatch)
  private
    FCP: IConnectionPoint; // these two used to connect and disconnect events
    FCookie: integer;
  protected
    // IInterface
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    // IDispatch
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DIID: Integer; const IID: TGUID;
      LocaleID: Integer; Flags: Word; var Params;
      VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  public
    constructor Create(Fitter: IDispatch);
    destructor Destroy; override;  
  end;

{ TTestFitterEventSink }

const // from Fitter_TLB
  DIID_IDMFitterEvents: TGUID = '{673AB003-9A0B-11D4-B2A4-FD6847C75367}';
  DISPID_OnProgress=201;
  DISPID_OnGetLMFunction=203;

constructor TTestFitterEventSink.Create(Fitter: IDispatch);
begin
  inherited Create;
  Assert(Assigned(Fitter)); // check parameter
  OleCheck((Fitter as 
    IConnectionPointContainer).FindConnectionPoint(DIID_IDMFitterEvents, FCP));
  if Assigned(FCP)
  then FCP.Advise(Self, FCookie);
end;

destructor TTestFitterEventSink.Destroy;
begin
  if Assigned(FCP)
  then FCP.Unadvise(FCookie);
  FCP:=nil;
  inherited;
end;

function TTestFitterEventSink.GetIDsOfNames(const IID: TGUID;
  Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result:=E_NOTIMPL;
end;

function TTestFitterEventSink.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
  Pointer(TypeInfo):=nil;
  Result:=E_NOTIMPL;
end;

function TTestFitterEventSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Count:=0;
  Result:=S_OK;
end;

// process several events defined in TLB
function TTestFitterEventSink.Invoke(DIID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
var
  X, Parameters: Variant;
  F: double;  
begin
  Result:=DISP_E_MEMBERNOTFOUND; 
  if ((Flags and DISPATCH_METHOD)<>0) and (DIID=DISPID_OnProgress) then
  begin
    Result:=S_OK;
    write('*');
    Sleep(100); // <- this is ONLY to slow down animation!!! should be removed
  end;
  if ((Flags and DISPATCH_METHOD)<>0) and (DIID=DISPID_OnGetLMFunction) 
    and (TDISPPARAMS(Params).cArgs=2) then
  begin
    Result:=S_OK;
    // warning: reverse order of arguments!
    VarCopyNoInd(X, Variant(TDISPPARAMS(Params).rgvarg[1]));
    VarCopyNoInd(Parameters, Variant(TDISPPARAMS(Params).rgvarg[0]));
    // calculate function (you may additionally check parameters here)
    F:=Parameters[0]+Parameters[1]*X+Parameters[2]*X*X+Parameters[3]*X*X*X;
    PVariant(VarResult)^:=F;
  end;
  {Here you can add OnGetLMDerivative with dispid 204, or replace the code
  above with your implementation of OnGetLinearBasis with dispid 202. 
  Beware of incorrect parameters processing!!!}
end;

function TTestFitterEventSink.QueryInterface(const IID: TGUID;
  out Obj): HResult;
var
  MyIID: TGUID;   
begin
  MyIID:=IID;
  if IsEqualIID(IID, DIID_IDMFitterEvents) 
  then MyIID:=StringToGuid('{00020400-0000-0000-C000-000000000046}');
  Result:=inherited QueryInterface(MyIID, Obj);  // ^ IID_IDispatch
end;

{ Main application code }

var
  X,Y,P,S,O,Fitter: variant;
  I: integer;
  Buffer: array[0..1024] of char;
  TestFitterEventSink: TTestFitterEventSink;

const
  VectorSize=100;
  ParamCount=4;
  
begin
  { TODO -oUser -cConsole Main : Insert code here }
  writeln('This application invokes DMFitter.LMFit()');
  writeln('Following expression will be fitted:');
  writeln('0.123+3.456*I-0.345*I*I+0.00567*I*I*I');
  writeln;
  // prepare fitting data
  X:=VarArrayCreate([0, VectorSize-1], varVariant);
  Y:=VarArrayCreate([0, VectorSize-1], varVariant);
  for I:=0 to VectorSize-1 do
  begin
    X[I]:=I;
    Y[I]:=0.123+3.456*I-0.345*I*I+0.00567*I*I*I;
  end;
  // initialize parameters and fix flags
  P:=VarArrayCreate([0, ParamCount-1], varVariant);
  S:=VarArrayCreate([0, ParamCount-1], varVariant);
  for I:=0 to ParamCount-1 do // as in TestLMFit.htm...
  begin
    P[I]:=1;
    S[I]:=0;
  end;
  // initialize fitter options
  O:=VarArrayCreate([0, 2], varVariant);
  O[0]:=1e-7; // Parameters convergence threshold
  O[1]:=1e-7; // Chi^2 convergence threshold
  O[2]:=0; // derivative dX (0 means default value)
  // create and initialize fitter object
  OleInitialize(nil);
  try
    Fitter:=CreateOLEObject('Fitter.DMFitter');
  except
    on E: Exception do
    begin
      writeln('Unable to create <Fitter.DMFitter>!');
      StrPCopy(Buffer, E.Message);
      ANSIToOem(Buffer,Buffer); // convert codepage for console
      writeln(Buffer);
      writeln('Click <ENTER> to close');
      readln;
      halt;
    end;
  end;
  Fitter.Expression:=-2; // means using ONLY OnGetLMFunction 
  Fitter.ParamCount:=ParamCount; // number of terms in the polynom
  Fitter.X:=X;
  Fitter.Y:=Y;
  Fitter.Parameters:=P;
  Fitter.Sigmas:=S;
  Fitter.Iterations:=30;
  Fitter.Options:=O; // <- this step is optional
  Fitter.WeightType:=0; // <- this step is optional
  // create event sink
  TestFitterEventSink:=TTestFitterEventSink.Create(Fitter);
  // run fitting
  if Fitter.LMFit then
  begin // print results
    writeln; // after progress output
    writeln('Fitter.ResultCode: ', Fitter.ResultCode);
    writeln('Fitter.Deviation: ', Fitter.Deviation);
    writeln('Fitter.Iterations: ', Fitter.Iterations);
    P:=Fitter.Parameters;
    for I:=0 to Fitter.ParamCount-1 do
    writeln('Parameters[', I, ']=', P[I]);
  end else
    writeln('Fitting error #', Fitter.ResultCode);
  // wait to close application
  writeln('Click <ENTER> to close');
  readln;
end.
